# Stores arrays in a dbm database
# By Evan Jones <ejones@uwaterloo.ca>
# http://www.eng.uwaterloo.ca/~ejones/
#
# Released under the BSD Licence:
# - Essentially, I hold copyright but you can do whatever you want with this,
#	as long as you acknowledge my copyright.
# http://www.opensource.org/licenses/bsd-license.html

package DBMTable;

# This class wraps a DBM class, overloading the FETCH, STORE and TIEHASH methods
@ISA = qw( SDBM_File );

use strict;
use Fcntl;
use SDBM_File;

# Opening the table
# arguments: filename, flags, permissions, a la DBM tie
sub TIEHASH( @ )
{
	my $package = 'DBMTable';
	my $filename = '';
	my $flags = O_RDWR;
	my $permissions = 0640;
	$package = $_[0] if ( $#_ >= 0 );
	$filename = $_[1] if ( $#_ >= 1 );
	$flags = $_[2] if ( $#_ >= 2 );
	$permissions = $_[3] if ( $#_ >= 3 );
		
	my $obj = SDBM_File::TIEHASH( 'SDBM_File', $filename, $flags, $permissions )
		or return 0;
	
	return bless( $obj, $package );
}

sub FETCH( \%$ )
{
	my ( $this, $key ) = @_;
	
	my $string = $this->SUPER::FETCH( $key );

	# The -1 forces the split to keep null fields
	my @value = split( /;(?!\\)/, $string, -1 );
	
	# Unescape semicolons
	map( s/;\\/;/, @value );
	
	return \@value;
}

sub STORE( $$$ )
{
	my ( $this, $key, $value ) = @_;

	die "Must store array references" if ( ref( $value ) ne 'ARRAY' );
	
	
	# Escape semicolons
	map( s/;/;\\/, @$value );
	my $string = join( ';', @$value );
	
	$this->SUPER::STORE( $key, $string );
}

# Creates a new DBM database with the following structure:
# Keys are numeric integers, starting at one
# Key zero stores the heading names
# Returns true if sucessful
sub create( $$ )
{
	my ( $filename, $headings ) = ( @_ );
	
	return 0 if ( -e $filename );
	
	tie( my %table, 'DBMTable', $filename, O_RDWR | O_CREAT, 0640 );
		
	$table{0} = $headings;
	
	untie( %table );
	
	return 1;
}

1;