#!/usr/bin/perl -w

use strict;

use FindBin;
use lib "$FindBin::Bin";

use DBMTable;

if ( @ARGV != 1 && @ARGV != 2 )
{
	print <<EOF;
dbmbase-dumper.pl: Dumps a DBMTable to semicolon seperated values, or recreates a table from such a text file
Usage:
	Export: ./dbmbase-dumper.pl <dbm file name>
	Import: ./dbmbase-dumper.pl <input text file> <dbm file name>
EOF
	exit 1;
}

if ( @ARGV == 1 )
{
	my %info;
	if ( tie( %info, 'DBMTable', $ARGV[0] ) )
	{
		while ( my ($key,$value) = each %info )
		{
			# Escape semicolons
			my $string = DBMTable::arrayToString( $value );
			print "$key;" . join( ';', @$value ) . "\n";
		}
	}
	else
	{
		print STDERR "Could not open $ARGV[0] as a DBMTable file\n";
	}
}
else
{
	if ( open( FH, $ARGV[0] ) )
	{
		my %db;
		foreach my $line ( <FH> )
		{
			if ( $line =~ /^([^;]+);(.*)$/ )
			{
				$db{$1} = DBMTable::stringToArray( $2 );
			}
		}
		close( FH ) or die;
		
		while ( my ($key,$value) = each %db )
		{
			# Escape semicolons
			my $string = DBMTable::arrayToString( $value );
		}
		
		# Create the table
		DBMTable::create( $ARGV[1], $db{0} );
		my %info;
		if ( tie( %info, 'DBMTable', $ARGV[1] ) )
		{
			my $count = 0;
			while ( my ($key,$value) = each %db )
			{
				next if $key == 0;
				
				$info{$key} = $value;
				$count ++;
			}
			
			untie( %info ) or die;
			print "$count records imported into $ARGV[1]\n";
		}
		else
		{
			print STDERR "Could not create DBMTable $ARGV[1]\n";
		}
	}
	else
	{
		print STDERR "Could not open $ARGV[0]\n";
	}
}